#include "stdafx.h"
#include "XReg.h"


XReg::XReg()
{
	m_hKey = NULL;
}

XReg::~XReg()
{
	Close();

}

BOOL XReg::Open(HKEY hParentKey, LPCTSTR szSubKeyName, REGSAM samDesired, BOOL bCreateIfNotExist)
{
	Close();

	LONG res = ::RegOpenKeyEx(hParentKey, szSubKeyName, 0, samDesired, &m_hKey);
	if (res != ERROR_SUCCESS)
	{
		if(bCreateIfNotExist)		//  .
		{
			return Create(hParentKey, szSubKeyName, samDesired);
		}

		//ASSERT(0);
		return FALSE;
	}

	return TRUE;
}

BOOL XReg::Create(HKEY hParentKey, LPCTSTR szSubKeyName, REGSAM samDesired)
{
	Close();

	DWORD dispositionReal;
	LONG res = ::RegCreateKeyEx(hParentKey, szSubKeyName, 0, NULL, REG_OPTION_NON_VOLATILE, samDesired, NULL,  &m_hKey, &dispositionReal);
	if (res != ERROR_SUCCESS)
	{
		ASSERT(0);
		return FALSE;
	}
	return TRUE;
}



void XReg::Close()
{
	if(m_hKey) RegCloseKey(m_hKey);
	m_hKey = NULL;
}

BOOL XReg::GetStr(LPCTSTR szValueName, CString& sResult, LPCTSTR szDefaultValue)
{
	if(szDefaultValue) sResult = szDefaultValue;		// default 

	if(m_hKey==NULL){ASSERT(0); return FALSE;}

	LONG		lResult;
	BYTE*		pBuf=NULL;
	DWORD		dwBufSize = 0;
	BOOL		ret=FALSE;
#ifdef _UNICODE
	DWORD		dwType = REG_MULTI_SZ;
#else
	DWORD		dwType = REG_SZ;
#endif

	// Ʈ   ´.
	lResult = RegQueryValueEx(m_hKey, szValueName, NULL, &dwType, pBuf, &dwBufSize);
	if(dwBufSize==0) goto END;
	pBuf = (BYTE*)malloc(dwBufSize*sizeof(TCHAR));
	if(pBuf==NULL) goto END;

	// ٽ о ´.
	lResult = RegQueryValueEx(m_hKey, szValueName, NULL, &dwType, pBuf, &dwBufSize);
	if(lResult!=ERROR_SUCCESS) goto END;

	sResult = (TCHAR*)pBuf;

	ret = TRUE;

END :
	if(pBuf) free(pBuf);
	return ret;
}

BOOL XReg::SetStr(LPCTSTR szValueName, LPCTSTR szValue)
{
	if(m_hKey==NULL){ASSERT(0); return FALSE;}

	USES_CONVERSION;
	const CHAR* _szValue = T2CA(szValue);

	if(::RegSetValueExA(m_hKey, T2CA(szValueName), NULL,REG_SZ, (const BYTE*)_szValue, DWORD(strlen(_szValue)+sizeof(CHAR)) )!=ERROR_SUCCESS)
	{
		ASSERT(0); return FALSE;
	}

	return TRUE;
}

BOOL XReg::SetBin(LPCTSTR szValueName, void* pData, int nDataLen)
{
	if(m_hKey==NULL){ASSERT(0); return FALSE;}

	if(RegSetValueEx(m_hKey, szValueName, NULL, REG_BINARY, (const BYTE*)pData, nDataLen)!=ERROR_SUCCESS)
	{
		ASSERT(0); return FALSE;
	}

	return TRUE;
}

BOOL XReg::SetStrW(LPCTSTR szValueName, LPCWSTR szValue)
{
	if(m_hKey==NULL){ASSERT(0); return FALSE;}

	USES_CONVERSION;
	const WCHAR* _szValue = T2CW(szValue);

	if(::RegSetValueEx(m_hKey,szValueName,NULL,REG_MULTI_SZ, (const BYTE*)_szValue, DWORD((lstrlenW(_szValue)+1)*sizeof(WCHAR)) )!=ERROR_SUCCESS)
	{
		ASSERT(0); return FALSE;
	}

	return TRUE;
}


////////////////////////////////////////////////////////////////////////////////////////////////////
///         Ư  Ű . NT Ű Ű   Ѵ
/// @param  
/// @return 
/// @date   Tuesday, April 22, 2008  8:22:41 AM
////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL XReg::DeleteSubKey(LPCTSTR subKeyName)
{
	if(m_hKey==NULL){ASSERT(0); return FALSE;}

	LONG res = RegDeleteKey(m_hKey, subKeyName);
	if (res != ERROR_SUCCESS)
	{
		ASSERT(0);
		return FALSE;
	}

	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////////////////
///         Ŀϰ Ű .
/// @param  
/// @return 
/// @date   Tuesday, April 22, 2008  8:23:05 AM
////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL XReg::RecurseDeleteKey(LPCTSTR subKeyName)
{
	XReg key;
	if(key.Open(m_hKey, subKeyName, KEY_READ | KEY_WRITE)==FALSE)
	{
		ASSERT(0);
		return FALSE;
	}

	FILETIME fileTime;
	const UINT32 kBufferSize = MAX_PATH + 1; // 256 in ATL
	DWORD size = kBufferSize;
	TCHAR buffer[kBufferSize];
	BOOL ret;
	while (RegEnumKeyEx(key.m_hKey, 0, buffer, &size, NULL, NULL, NULL, &fileTime) == ERROR_SUCCESS)
	{
		ret = key.RecurseDeleteKey(buffer);
		if (ret ==FALSE) 
			return ret;
		size = kBufferSize;
	}
	key.Close();
	return DeleteSubKey(subKeyName);
}


////////////////////////////////////////////////////////////////////////////////////////////////////
///           key  Ư value  
/// @param  
/// @return 
/// @date   Tuesday, April 22, 2008  8:21:42 AM
////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL XReg::DeleteValue(LPCTSTR szValueName)
{
	if(m_hKey==NULL){ASSERT(0); return FALSE;}

	LONG res = ::RegDeleteValue(m_hKey, szValueName);
	if(res!=ERROR_SUCCESS)
	{
		//ASSERT(0);
		return FALSE;
	}
	return TRUE;
}

BOOL XReg::GetInt(LPCTSTR szValueName, int& nResult, int nDefaultValue)
{
	return GetDWORD(szValueName, (DWORD&)nResult, (DWORD)nDefaultValue);
}

BOOL XReg::GetDWORD(LPCTSTR szValueName, DWORD& dwResult, DWORD dwDefaultValue)
{
	LONG		lResult;
	BYTE		pBuf[4];
	DWORD		dwBufSize = sizeof(DWORD);
	DWORD		dwType = REG_DWORD;

	dwResult = dwDefaultValue;

	// о ´.
	lResult = RegQueryValueEx(m_hKey, szValueName, NULL, &dwType, pBuf, &dwBufSize);
	if(lResult!=ERROR_SUCCESS) goto FAIL;

	dwResult = *((DWORD*)pBuf);
	return TRUE;

FAIL :
	return FALSE;
}

BOOL  XReg::GetBin(LPCTSTR szValueName, BYTE* pRetBuf, int nBufSize)
{
	LONG		lResult;
	DWORD		dwBufSize = nBufSize;
	DWORD		dwType = REG_BINARY;

	// о ´.
	lResult = RegQueryValueEx(m_hKey, szValueName, NULL, &dwType, pRetBuf, &dwBufSize);
	if(lResult!=ERROR_SUCCESS) {ASSERT(0); return FALSE;}
	nBufSize = dwBufSize;

	return TRUE;
}


BOOL  XReg::GetBinAlloc(LPCTSTR szValueName, BYTE** pRetBuf, int& nBufSize)
{
	LONG		lResult;
	DWORD		dwBufSize = nBufSize;
	DWORD		dwType = REG_BINARY;

	// ̳ʸ   ´.
	lResult = RegQueryValueEx(m_hKey, szValueName, NULL, &dwType, (BYTE*)*pRetBuf, &dwBufSize);
	if(dwBufSize==0) {ASSERT(0); return FALSE;}
	*pRetBuf = (BYTE*)malloc(dwBufSize*sizeof(BYTE));
	if(*pRetBuf==NULL) {ASSERT(0); return FALSE;}

	// о ´.
	lResult = RegQueryValueEx(m_hKey, szValueName, NULL, &dwType,(BYTE*) *pRetBuf, &dwBufSize);
	if(lResult!=ERROR_SUCCESS) {ASSERT(0); free(*pRetBuf); return FALSE;}
	nBufSize = dwBufSize;

	return TRUE;
}


BOOL XReg::SetInt(LPCTSTR szValueName, int nValue)
{
	return SetDWORD(szValueName, (DWORD)nValue);
}

BOOL XReg::SetDWORD(LPCTSTR szValueName, DWORD dwValue)
{
	LONG		lResult;
	lResult = ::RegSetValueEx(m_hKey,szValueName,NULL,REG_DWORD,(const BYTE*)&dwValue,sizeof(DWORD));
	if(lResult) return FALSE;
	return TRUE;
}

