#include "StdAfx.h"
#include "XRichEditCtrl.h"

class XRichEditCtrlDllLoader
{
public :
	XRichEditCtrlDllLoader()
	{
		m_hDll = ::LoadLibraryA("RichEd32.dll");
		if(m_hDll==NULL)
			m_hDll = ::LoadLibraryA("RichEd20.dll");
		if(m_hDll==NULL){ASSERT(0);}
	}
	~XRichEditCtrlDllLoader()
	{
		if(m_hDll) ::FreeLibrary(m_hDll);
	}
	HMODULE		m_hDll;
} richeddllLoader;


XRichEditCtrl::XRichEditCtrl(void)
{
	m_hWnd = NULL;
	m_hFont = NULL;
	m_clrDefault = RGB(0,0,0);
}

XRichEditCtrl::~XRichEditCtrl(void)
{
	if(m_hFont)
		DeleteObject(m_hFont);
}



////////////////////////////////////////////////////////////////////////////////////////////////////
///         
/// @param  
/// @return 
/// @date   Monday, March 24, 2008  1:42:02 AM
////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL XRichEditCtrl::Create(HINSTANCE hInstance, HWND hWndParent, int x, int y,  int width, int height)
{
	m_hWnd = CreateWindowEx(WS_EX_CLIENTEDGE, RICHEDIT_CLASS, NULL, 
		WS_CHILD | WS_VISIBLE | WS_VSCROLL | WS_TABSTOP | /*WS_BORDER |*/ ES_NOHIDESEL | ES_MULTILINE | ES_READONLY , 
		x, y, width, height, hWndParent, (HMENU)1, hInstance, NULL);
	ASSERT(m_hWnd);

	HFONT hSysFont = (HFONT)GetStockObject(DEFAULT_GUI_FONT);
	LOGFONT lfont;
	GetObject(hSysFont, sizeof(lfont), &lfont);
	m_hFont = ::CreateFontIndirect(&lfont);
	SendMessage(m_hWnd, WM_SETFONT, (WPARAM)m_hFont, 0);

	return TRUE;
}

void XRichEditCtrl::Test()
{
	SetTextColor(RGB(1,200,3));
	AddText(_T("Hello\r\n"));
	AddFormatText(_T("@@rRed@@gGreen@@bBlue @@wWhie@@kBlack \r\n"));
}

int XRichEditCtrl::GetWindowTextLength()
{
	return ::GetWindowTextLength(m_hWnd);
}

void XRichEditCtrl::SetSel(long nStartChar, long nEndChar)
{
	ASSERT(::IsWindow(m_hWnd));
	CHARRANGE cr;
	cr.cpMin = nStartChar;
	cr.cpMax = nEndChar;
	::SendMessage(m_hWnd, EM_EXSETSEL, 0, (LPARAM)&cr);
}

void XRichEditCtrl::AddText(CString strText)
{
	int	iStartPos = 0;
	int	iEndPos   = GetWindowTextLength();

	iStartPos = iEndPos;

	SetSel(iStartPos, iEndPos);
	ReplaceSel((LPCTSTR) strText);
	HideSelection(FALSE, TRUE);
}

////////////////////////////////////////////////////////////////////////////////////////////////////
///         ִ ؽƮ ߰ϱ
/// @param  
/// @return 
/// @date   Monday, March 24, 2008  2:14:48 AM
////////////////////////////////////////////////////////////////////////////////////////////////////
void XRichEditCtrl::AddFormatText(CString sText)
{
	const TCHAR* p = sText;
	CString			sTemp;

#define FLUSH_STR if(sTemp.IsEmpty()==FALSE) {AddText(sTemp); sTemp.Empty(); }

	while(*p)
	{
		if(*p=='@' && *(p+1)=='@' && *(p+2) )
		{
			FLUSH_STR;
			switch(*(p+2))
			{
			case 'r' : SetTextColor(RGB(255,0,0)); break;
			case 'g' : SetTextColor(RGB(0,255,0)); break;
			case 'b' : SetTextColor(RGB(0,0,255)); break;
			case 'w' : SetTextColor(RGB(255,255,255)); break;
			case 'k' : SetTextColor(RGB(0,0,0)); break;
			default :
				ASSERT(0);
			}
			p+=3;
			continue;
		}
		sTemp += *p;
		p++;
	}
	FLUSH_STR;
}

void XRichEditCtrl::AddFormatTextLn(CString sText)
{
	AddFormatText(sText);
	AddText(_T("\r\n"));
	SetTextColor(m_clrDefault);		//   
}

void XRichEditCtrl::SetTextColor(COLORREF clr)
{
	CHARFORMAT cf;
	ZeroMemory(&cf, sizeof(CHARFORMAT));

	cf.cbSize      = sizeof(CHARFORMAT);
	cf.dwMask      = CFM_COLOR;
	cf.crTextColor = clr;

	SetSelectionCharFormat(cf);
}

void XRichEditCtrl::SetDefaultColor()
{
	SetTextColor(m_clrDefault);
}


void XRichEditCtrl::SetBold(BOOL bBold)
{
	CHARFORMAT cf;
	ZeroMemory(&cf, sizeof(CHARFORMAT));

	cf.cbSize      = sizeof(CHARFORMAT);
	cf.dwMask      = CFM_BOLD;
	cf.dwEffects   = (DWORD) (bBold ? CFE_BOLD : ~CFE_BOLD);

	SetSelectionCharFormat(cf);
}

void XRichEditCtrl::SetSize(int nSize)
{
	CHARFORMAT cf;
	ZeroMemory(&cf, sizeof(CHARFORMAT));

	cf.cbSize      = sizeof(CHARFORMAT);
	cf.dwMask      = CFM_SIZE;
	cf.yHeight	   = nSize;

	SetSelectionCharFormat(cf);
}


BOOL XRichEditCtrl::SetSelectionCharFormat(CHARFORMAT &cf)
{
	ASSERT(::IsWindow(m_hWnd));
	cf.cbSize = sizeof(CHARFORMAT);
	return (BOOL)::SendMessage(m_hWnd, EM_SETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);
}

